# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v20.enums.types import (
    sk_ad_network_coarse_conversion_value,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "CustomerSkAdNetworkConversionValueSchema",
    },
)


class CustomerSkAdNetworkConversionValueSchema(proto.Message):
    r"""A CustomerSkAdNetworkConversionValueSchema.

    Attributes:
        resource_name (str):
            Output only. The resource name of the schema.
            CustomerSkAdNetworkConversionValueSchema resource names have
            the form:
            customers/{customer_id}/customerSkAdNetworkConversionValueSchemas/{account_link_id}
        schema (google.ads.googleads.v20.resources.types.CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema):
            Output only. The schema for the specified
            resource.
    """

    class SkAdNetworkConversionValueSchema(proto.Message):
        r"""The CustomerLink specific SkAdNetworkConversionValueSchema.

        Attributes:
            app_id (str):
                Required. Output only. Apple App Store app
                ID.
            measurement_window_hours (int):
                Output only. A time window (measured in hours) post-install,
                after which the App Attribution Partner or advertiser stops
                calling [updateConversionValue]
                (https://developer.apple.com/documentation/storekit/skadnetwork/3566697-updateconversionvalue).
            fine_grained_conversion_value_mappings (MutableSequence[google.ads.googleads.v20.resources.types.CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.FineGrainedConversionValueMappings]):
                Output only. Fine grained conversion value
                mappings. For SkAdNetwork versions >= 4.0 that
                support multiple conversion windows, fine
                grained conversion value mappings are only
                applicable to the first postback.
            postback_mappings (MutableSequence[google.ads.googleads.v20.resources.types.CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.PostbackMapping]):
                Output only. Per-postback conversion value
                mappings for postbacks in multiple conversion
                windows. Only applicable for SkAdNetwork
                versions >= 4.0.
        """

        class FineGrainedConversionValueMappings(proto.Message):
            r"""Mappings for fine grained conversion value.

            Attributes:
                fine_grained_conversion_value (int):
                    Output only. Fine grained conversion value. Valid values are
                    in the inclusive range [0,63].
                conversion_value_mapping (google.ads.googleads.v20.resources.types.CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.ConversionValueMapping):
                    Output only. Conversion events the fine
                    grained conversion value maps to.
            """

            fine_grained_conversion_value: int = proto.Field(
                proto.INT32,
                number=1,
            )
            conversion_value_mapping: "CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.ConversionValueMapping" = proto.Field(
                proto.MESSAGE,
                number=2,
                message="CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.ConversionValueMapping",
            )

        class PostbackMapping(proto.Message):
            r"""Mappings for each postback in multiple conversion windows.

            This message has `oneof`_ fields (mutually exclusive fields).
            For each oneof, at most one member field can be set at the same time.
            Setting any member of the oneof automatically clears all other
            members.

            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

            Attributes:
                postback_sequence_index (int):
                    Output only. 0-based index that indicates the order of
                    postback. Valid values are in the inclusive range [0,2].
                coarse_grained_conversion_value_mappings (google.ads.googleads.v20.resources.types.CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.CoarseGrainedConversionValueMappings):
                    Output only. Conversion value mappings for
                    all coarse grained conversion values.
                lock_window_coarse_conversion_value (google.ads.googleads.v20.enums.types.SkAdNetworkCoarseConversionValueEnum.SkAdNetworkCoarseConversionValue):
                    Output only. Coarse grained conversion value
                    that triggers conversion window lock.

                    This field is a member of `oneof`_ ``lock_window_trigger``.
                lock_window_fine_conversion_value (int):
                    Output only. Fine grained conversion value
                    that triggers conversion window lock.

                    This field is a member of `oneof`_ ``lock_window_trigger``.
                lock_window_event (str):
                    Output only. Event name that triggers
                    conversion window lock.

                    This field is a member of `oneof`_ ``lock_window_trigger``.
            """

            postback_sequence_index: int = proto.Field(
                proto.INT32,
                number=1,
            )
            coarse_grained_conversion_value_mappings: "CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.CoarseGrainedConversionValueMappings" = proto.Field(
                proto.MESSAGE,
                number=2,
                message="CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.CoarseGrainedConversionValueMappings",
            )
            lock_window_coarse_conversion_value: (
                sk_ad_network_coarse_conversion_value.SkAdNetworkCoarseConversionValueEnum.SkAdNetworkCoarseConversionValue
            ) = proto.Field(
                proto.ENUM,
                number=3,
                oneof="lock_window_trigger",
                enum=sk_ad_network_coarse_conversion_value.SkAdNetworkCoarseConversionValueEnum.SkAdNetworkCoarseConversionValue,
            )
            lock_window_fine_conversion_value: int = proto.Field(
                proto.INT32,
                number=4,
                oneof="lock_window_trigger",
            )
            lock_window_event: str = proto.Field(
                proto.STRING,
                number=5,
                oneof="lock_window_trigger",
            )

        class CoarseGrainedConversionValueMappings(proto.Message):
            r"""Mappings for coarse grained conversion values.

            Attributes:
                low_conversion_value_mapping (google.ads.googleads.v20.resources.types.CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.ConversionValueMapping):
                    Output only. Mapping for "low" coarse
                    conversion value.
                medium_conversion_value_mapping (google.ads.googleads.v20.resources.types.CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.ConversionValueMapping):
                    Output only. Mapping for "medium" coarse
                    conversion value.
                high_conversion_value_mapping (google.ads.googleads.v20.resources.types.CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.ConversionValueMapping):
                    Output only. Mapping for "high" coarse
                    conversion value.
            """

            low_conversion_value_mapping: "CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.ConversionValueMapping" = proto.Field(
                proto.MESSAGE,
                number=1,
                message="CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.ConversionValueMapping",
            )
            medium_conversion_value_mapping: "CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.ConversionValueMapping" = proto.Field(
                proto.MESSAGE,
                number=2,
                message="CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.ConversionValueMapping",
            )
            high_conversion_value_mapping: "CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.ConversionValueMapping" = proto.Field(
                proto.MESSAGE,
                number=3,
                message="CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.ConversionValueMapping",
            )

        class ConversionValueMapping(proto.Message):
            r"""Represents mapping from one conversion value to one or more
            conversion events.

            Attributes:
                min_time_post_install_hours (int):
                    Output only. The minimum of the time range in
                    which a user was last active during the
                    measurement window.
                max_time_post_install_hours (int):
                    Output only. The maximum of the time range in
                    which a user was last active during the
                    measurement window.
                mapped_events (MutableSequence[google.ads.googleads.v20.resources.types.CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.Event]):
                    Output only. The conversion value may be
                    mapped to multiple events with various
                    attributes.
            """

            min_time_post_install_hours: int = proto.Field(
                proto.INT64,
                number=1,
            )
            max_time_post_install_hours: int = proto.Field(
                proto.INT64,
                number=2,
            )
            mapped_events: MutableSequence[
                "CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.Event"
            ] = proto.RepeatedField(
                proto.MESSAGE,
                number=3,
                message="CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.Event",
            )

        class Event(proto.Message):
            r"""Defines a Google conversion event that the conversion value
            is mapped to.

            This message has `oneof`_ fields (mutually exclusive fields).
            For each oneof, at most one member field can be set at the same time.
            Setting any member of the oneof automatically clears all other
            members.

            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

            Attributes:
                mapped_event_name (str):
                    Output only. Google event name represented by
                    this conversion value.
                currency_code (str):
                    Output only. The reported currency for the event_revenue.
                    ISO 4217 three-letter currency code, for example, "USD".
                event_revenue_range (google.ads.googleads.v20.resources.types.CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.Event.RevenueRange):
                    Output only. The event revenue range.

                    This field is a member of `oneof`_ ``revenue_rate``.
                event_revenue_value (float):
                    Output only. The specific event revenue
                    value.

                    This field is a member of `oneof`_ ``revenue_rate``.
                event_occurrence_range (google.ads.googleads.v20.resources.types.CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.Event.EventOccurrenceRange):
                    Output only. The event counter range.

                    This field is a member of `oneof`_ ``event_rate``.
                event_counter (int):
                    Output only. For specific event counter
                    values.

                    This field is a member of `oneof`_ ``event_rate``.
            """

            class RevenueRange(proto.Message):
                r"""Defines a range for revenue values.

                Attributes:
                    min_event_revenue (float):
                        Output only. For revenue ranges, the minimum value in
                        ``currency_code`` for which this conversion value would be
                        updated. A value of 0 will be treated as unset.
                    max_event_revenue (float):
                        Output only. For revenue ranges, the maximum value in
                        ``currency_code`` for which this conversion value would be
                        updated. A value of 0 will be treated as unset.
                """

                min_event_revenue: float = proto.Field(
                    proto.DOUBLE,
                    number=3,
                )
                max_event_revenue: float = proto.Field(
                    proto.DOUBLE,
                    number=4,
                )

            class EventOccurrenceRange(proto.Message):
                r"""Defines a range for event counter values.

                Attributes:
                    min_event_count (int):
                        Output only. For event counter ranges, the
                        minimum of the defined range. A value of 0 will
                        be treated as unset.
                    max_event_count (int):
                        Output only. For event counter ranges, the
                        maximum of the defined range. A value of 0 will
                        be treated as unset.
                """

                min_event_count: int = proto.Field(
                    proto.INT64,
                    number=1,
                )
                max_event_count: int = proto.Field(
                    proto.INT64,
                    number=2,
                )

            mapped_event_name: str = proto.Field(
                proto.STRING,
                number=1,
            )
            currency_code: str = proto.Field(
                proto.STRING,
                number=2,
            )
            event_revenue_range: "CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.Event.RevenueRange" = proto.Field(
                proto.MESSAGE,
                number=3,
                oneof="revenue_rate",
                message="CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.Event.RevenueRange",
            )
            event_revenue_value: float = proto.Field(
                proto.DOUBLE,
                number=4,
                oneof="revenue_rate",
            )
            event_occurrence_range: "CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.Event.EventOccurrenceRange" = proto.Field(
                proto.MESSAGE,
                number=5,
                oneof="event_rate",
                message="CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.Event.EventOccurrenceRange",
            )
            event_counter: int = proto.Field(
                proto.INT64,
                number=6,
                oneof="event_rate",
            )

        app_id: str = proto.Field(
            proto.STRING,
            number=1,
        )
        measurement_window_hours: int = proto.Field(
            proto.INT32,
            number=2,
        )
        fine_grained_conversion_value_mappings: MutableSequence[
            "CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.FineGrainedConversionValueMappings"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=3,
            message="CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.FineGrainedConversionValueMappings",
        )
        postback_mappings: MutableSequence[
            "CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.PostbackMapping"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=4,
            message="CustomerSkAdNetworkConversionValueSchema.SkAdNetworkConversionValueSchema.PostbackMapping",
        )

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    schema: SkAdNetworkConversionValueSchema = proto.Field(
        proto.MESSAGE,
        number=2,
        message=SkAdNetworkConversionValueSchema,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Gets specific details about the most recent changes in the given account.

Changes include the name of the field that changed, and both the old and new
values.
"""

import argparse
from datetime import datetime, timedelta
import sys
from typing import Any

from proto.enums import ProtoEnumMeta

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.util import get_nested_attr
from google.ads.googleads.v22.services.services.google_ads_service.client import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    SearchGoogleAdsRequest,
    SearchPagedResponse,
    GoogleAdsRow,
)
from google.ads.googleads.v22.resources.types.change_event import ChangeEvent


# [START get_change_details]
def main(client: GoogleAdsClient, customer_id: str) -> None:
    """Gets specific details about the most recent changes in the given account.

    Args:
      client: The Google Ads client.
      customer_id: The Google Ads customer ID.
    """
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )

    # Construct a query to find details for recent changes in your account.
    # The LIMIT clause is required for the change_event resource.
    # The maximum size is 10000, but a low limit was set here for demonstrative
    # purposes. For more information see:
    # https://developers.google.com/google-ads/api/docs/change-event#getting_changes
    # The WHERE clause on change_date_time is also required. It must specify a
    # window within the past 30 days.
    tomorrow: str = (datetime.now() + timedelta(1)).strftime("%Y-%m-%d")
    two_weeks_ago: str = (datetime.now() + timedelta(-14)).strftime("%Y-%m-%d")
    query: str = f"""
        SELECT
          change_event.resource_name,
          change_event.change_date_time,
          change_event.change_resource_name,
          change_event.user_email,
          change_event.client_type,
          change_event.change_resource_type,
          change_event.old_resource,
          change_event.new_resource,
          change_event.resource_change_operation,
          change_event.changed_fields
        FROM change_event
        WHERE change_event.change_date_time <= '{tomorrow}'
        AND change_event.change_date_time >= '{two_weeks_ago}'
        ORDER BY change_event.change_date_time DESC
        LIMIT 5"""

    search_request: SearchGoogleAdsRequest = client.get_type(
        "SearchGoogleAdsRequest"
    )
    search_request.customer_id = customer_id
    search_request.query = query

    results: SearchPagedResponse = googleads_service.search(
        request=search_request
    )

    row: GoogleAdsRow
    for row in results:
        event: ChangeEvent = row.change_event
        resource_type: str = event.change_resource_type.name
        old_resource: Any
        new_resource: Any
        if resource_type == "AD":
            old_resource = event.old_resource.ad
            new_resource = event.new_resource.ad
        elif resource_type == "AD_GROUP":
            old_resource = event.old_resource.ad_group
            new_resource = event.new_resource.ad_group
        elif resource_type == "AD_GROUP_AD":
            old_resource = event.old_resource.ad_group_ad
            new_resource = event.new_resource.ad_group_ad
        elif resource_type == "AD_GROUP_ASSET":
            old_resource = event.old_resource.ad_group_asset
            new_resource = event.new_resource.ad_group_asset
        elif resource_type == "AD_GROUP_CRITERION":
            old_resource = event.old_resource.ad_group_criterion
            new_resource = event.new_resource.ad_group_criterion
        elif resource_type == "AD_GROUP_BID_MODIFIER":
            old_resource = event.old_resource.ad_group_bid_modifier
            new_resource = event.new_resource.ad_group_bid_modifier
        elif resource_type == "AD_GROUP_FEED":
            old_resource = event.old_resource.ad_group_feed
            new_resource = event.new_resource.ad_group_feed
        elif resource_type == "ASSET":
            old_resource = event.old_resource.asset
            new_resource = event.new_resource.asset
        elif resource_type == "ASSET_SET":
            old_resource = event.old_resource.asset_set
            new_resource = event.new_resource.asset_set
        elif resource_type == "ASSET_SET_ASSET":
            old_resource = event.old_resource.asset_set_asset
            new_resource = event.new_resource.asset_set_asset
        elif resource_type == "CAMPAIGN":
            old_resource = event.old_resource.campaign
            new_resource = event.new_resource.campaign
        elif resource_type == "CAMPAIGN_ASSET":
            old_resource = event.old_resource.campaign_asset
            new_resource = event.new_resource.campaign_asset
        elif resource_type == "CAMPAIGN_ASSET_SET":
            old_resource = event.old_resource.campaign_asset_set
            new_resource = event.new_resource.campaign_asset_set
        elif resource_type == "CAMPAIGN_BUDGET":
            old_resource = event.old_resource.campaign_budget
            new_resource = event.new_resource.campaign_budget
        elif resource_type == "CAMPAIGN_CRITERION":
            old_resource = event.old_resource.campaign_criterion
            new_resource = event.new_resource.campaign_criterion
        elif resource_type == "CAMPAIGN_FEED":
            old_resource = event.old_resource.campaign_feed
            new_resource = event.new_resource.campaign_feed
        elif resource_type == "CUSTOMER_ASSET":
            old_resource = event.old_resource.customer_asset
            new_resource = event.new_resource.customer_asset
        elif resource_type == "FEED":
            old_resource = event.old_resource.feed
            new_resource = event.new_resource.feed
        elif resource_type == "FEED_ITEM":
            old_resource = event.old_resource.feed_item
            new_resource = event.new_resource.feed_item
        else:
            print(
                "Unknown change_resource_type: '{event.change_resource_type}'"
            )
            # If the resource type is unrecognized then we continue to
            # the next row.
            continue

        print(
            f"On {event.change_date_time}, user {event.user_email} "
            f"used interface {event.client_type.name} to perform a(n) "
            f"{event.resource_change_operation.name} operation on a "
            f"{event.change_resource_type.name} with resource name "
            f"'{event.change_resource_name}'"
        )

        operation_type: str = event.resource_change_operation.name

        if operation_type in ("UPDATE", "CREATE"):
            for changed_field_path in event.changed_fields.paths:
                changed_field: str = changed_field_path
                # Change field name from "type" to "type_" so that it doesn't
                # raise an exception when accessed on the protobuf object, see:
                # https://developers.google.com/google-ads/api/docs/client-libs/python/library-version-10#field_names_that_are_reserved_words
                if changed_field == "type":
                    changed_field = "type_"

                new_value: Any = get_nested_attr(new_resource, changed_field)
                # If the field value is an Enum get the human readable name
                # so that it is printed instead of the field ID integer.
                if isinstance(type(new_value), ProtoEnumMeta):
                    new_value = new_value.name

                if operation_type == "CREATE":
                    print(f"\t{changed_field} set to {new_value}")
                else:
                    old_value: Any = get_nested_attr(
                        old_resource, changed_field
                    )
                    # If the field value is an Enum get the human readable name
                    # so that it is printed instead of the field ID integer.
                    if isinstance(type(old_value), ProtoEnumMeta):
                        old_value = old_value.name

                    print(
                        f"\t{changed_field} changed from {old_value} to {new_value}"
                    )
                    # [END get_change_details]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="This example gets specific details about the most recent "
        "changes in the given account."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")
    try:
        main(googleads_client, args.customer_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "BiddingErrorEnum",
    },
)


class BiddingErrorEnum(proto.Message):
    r"""Container for enum describing possible bidding errors."""

    class BiddingError(proto.Enum):
        r"""Enum describing possible bidding errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            BIDDING_STRATEGY_TRANSITION_NOT_ALLOWED (2):
                Cannot transition to new bidding strategy.
            CANNOT_ATTACH_BIDDING_STRATEGY_TO_CAMPAIGN (7):
                Cannot attach bidding strategy to campaign.
            INVALID_ANONYMOUS_BIDDING_STRATEGY_TYPE (10):
                Bidding strategy is not supported or cannot
                be used as anonymous.
            INVALID_BIDDING_STRATEGY_TYPE (14):
                The type does not match the named strategy's
                type.
            INVALID_BID (17):
                The bid is invalid.
            BIDDING_STRATEGY_NOT_AVAILABLE_FOR_ACCOUNT_TYPE (18):
                Bidding strategy is not available for the
                account type.
            CANNOT_CREATE_CAMPAIGN_WITH_BIDDING_STRATEGY (21):
                Campaign can not be created with given
                bidding strategy. It can be transitioned to the
                strategy, once eligible.
            CANNOT_TARGET_CONTENT_NETWORK_ONLY_WITH_CAMPAIGN_LEVEL_POP_BIDDING_STRATEGY (23):
                Cannot target content network only as
                campaign uses Page One Promoted bidding
                strategy.
            BIDDING_STRATEGY_NOT_SUPPORTED_WITH_AD_SCHEDULE (24):
                Budget Optimizer and Target Spend bidding
                strategies are not supported for campaigns with
                AdSchedule targeting.
            PAY_PER_CONVERSION_NOT_AVAILABLE_FOR_CUSTOMER (25):
                Pay per conversion is not available to all
                the customer, only few customers on the
                allow-list can use this.
            PAY_PER_CONVERSION_NOT_ALLOWED_WITH_TARGET_CPA (26):
                Pay per conversion is not allowed with Target
                CPA.
            BIDDING_STRATEGY_NOT_ALLOWED_FOR_SEARCH_ONLY_CAMPAIGNS (27):
                Cannot set bidding strategy to Manual CPM for
                search network only campaigns.
            BIDDING_STRATEGY_NOT_SUPPORTED_IN_DRAFTS_OR_EXPERIMENTS (28):
                The bidding strategy is not supported for use
                in drafts or experiments.
            BIDDING_STRATEGY_TYPE_DOES_NOT_SUPPORT_PRODUCT_TYPE_ADGROUP_CRITERION (29):
                Bidding strategy type does not support
                product type ad group criterion.
            BID_TOO_SMALL (30):
                Bid amount is too small.
            BID_TOO_BIG (31):
                Bid amount is too big.
            BID_TOO_MANY_FRACTIONAL_DIGITS (32):
                Bid has too many fractional digit precision.
            INVALID_DOMAIN_NAME (33):
                Invalid domain name specified.
            NOT_COMPATIBLE_WITH_PAYMENT_MODE (34):
                The field is not compatible with the payment
                mode.
            BIDDING_STRATEGY_TYPE_INCOMPATIBLE_WITH_SHARED_BUDGET (37):
                Bidding strategy type is incompatible with
                shared budget.
            BIDDING_STRATEGY_AND_BUDGET_MUST_BE_ALIGNED (38):
                The attached bidding strategy and budget must
                be aligned with each other if alignment is
                specified on either entity.
            BIDDING_STRATEGY_AND_BUDGET_MUST_BE_ATTACHED_TO_THE_SAME_CAMPAIGNS_TO_ALIGN (39):
                The attached bidding strategy and budget must
                be attached to the same campaigns to become
                aligned.
            BIDDING_STRATEGY_AND_BUDGET_MUST_BE_REMOVED_TOGETHER (40):
                The aligned bidding strategy and budget must
                be removed at the same time.
            CPC_BID_FLOOR_MICROS_GREATER_THAN_CPC_BID_CEILING_MICROS (41):
                cpc_bid_floor_micros is greater than cpc_bid_ceiling_micros.
            TARGET_ROAS_TOLERANCE_PERCENT_MILLIS_MUST_BE_INTEGER (42):
                target_roas_tolerance_percent_millis must be integer.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        BIDDING_STRATEGY_TRANSITION_NOT_ALLOWED = 2
        CANNOT_ATTACH_BIDDING_STRATEGY_TO_CAMPAIGN = 7
        INVALID_ANONYMOUS_BIDDING_STRATEGY_TYPE = 10
        INVALID_BIDDING_STRATEGY_TYPE = 14
        INVALID_BID = 17
        BIDDING_STRATEGY_NOT_AVAILABLE_FOR_ACCOUNT_TYPE = 18
        CANNOT_CREATE_CAMPAIGN_WITH_BIDDING_STRATEGY = 21
        CANNOT_TARGET_CONTENT_NETWORK_ONLY_WITH_CAMPAIGN_LEVEL_POP_BIDDING_STRATEGY = (
            23
        )
        BIDDING_STRATEGY_NOT_SUPPORTED_WITH_AD_SCHEDULE = 24
        PAY_PER_CONVERSION_NOT_AVAILABLE_FOR_CUSTOMER = 25
        PAY_PER_CONVERSION_NOT_ALLOWED_WITH_TARGET_CPA = 26
        BIDDING_STRATEGY_NOT_ALLOWED_FOR_SEARCH_ONLY_CAMPAIGNS = 27
        BIDDING_STRATEGY_NOT_SUPPORTED_IN_DRAFTS_OR_EXPERIMENTS = 28
        BIDDING_STRATEGY_TYPE_DOES_NOT_SUPPORT_PRODUCT_TYPE_ADGROUP_CRITERION = (
            29
        )
        BID_TOO_SMALL = 30
        BID_TOO_BIG = 31
        BID_TOO_MANY_FRACTIONAL_DIGITS = 32
        INVALID_DOMAIN_NAME = 33
        NOT_COMPATIBLE_WITH_PAYMENT_MODE = 34
        BIDDING_STRATEGY_TYPE_INCOMPATIBLE_WITH_SHARED_BUDGET = 37
        BIDDING_STRATEGY_AND_BUDGET_MUST_BE_ALIGNED = 38
        BIDDING_STRATEGY_AND_BUDGET_MUST_BE_ATTACHED_TO_THE_SAME_CAMPAIGNS_TO_ALIGN = (
            39
        )
        BIDDING_STRATEGY_AND_BUDGET_MUST_BE_REMOVED_TOGETHER = 40
        CPC_BID_FLOOR_MICROS_GREATER_THAN_CPC_BID_CEILING_MICROS = 41
        TARGET_ROAS_TOLERANCE_PERCENT_MILLIS_MUST_BE_INTEGER = 42


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.common.types import metric_goal
from google.ads.googleads.v21.enums.types import async_action_status
from google.ads.googleads.v21.enums.types import experiment_status
from google.ads.googleads.v21.enums.types import experiment_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "Experiment",
    },
)


class Experiment(proto.Message):
    r"""A Google ads experiment for users to experiment changes on
    multiple campaigns, compare the performance, and apply the
    effective changes.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the experiment. Experiment
            resource names have the form:

            ``customers/{customer_id}/experiments/{experiment_id}``
        experiment_id (int):
            Output only. The ID of the experiment. Read
            only.

            This field is a member of `oneof`_ ``_experiment_id``.
        name (str):
            Required. The name of the experiment. It must
            have a minimum length of 1 and maximum length of
            1024. It must be unique under a customer.
        description (str):
            The description of the experiment. It must
            have a minimum length of 1 and maximum length of
            2048.
        suffix (str):
            For system managed experiments, the
            advertiser must provide a suffix during
            construction, in the setup stage before moving
            to initiated. The suffix will be appended to the
            in-design and experiment campaign names so that
            the name is base campaign name + suffix.
        type_ (google.ads.googleads.v21.enums.types.ExperimentTypeEnum.ExperimentType):
            Required. The product/feature that uses this
            experiment.
        status (google.ads.googleads.v21.enums.types.ExperimentStatusEnum.ExperimentStatus):
            The Advertiser-chosen status of this
            experiment.
        start_date (str):
            Date when the experiment starts. By default,
            the experiment starts now or on the campaign's
            start date, whichever is later. If this field is
            set, then the experiment starts at the beginning
            of the specified date in the customer's time
            zone.

            Format: YYYY-MM-DD
            Example: 2019-03-14

            This field is a member of `oneof`_ ``_start_date``.
        end_date (str):
            Date when the experiment ends. By default,
            the experiment ends on the campaign's end date.
            If this field is set, then the experiment ends
            at the end of the specified date in the
            customer's time zone.

            Format: YYYY-MM-DD
            Example: 2019-04-18

            This field is a member of `oneof`_ ``_end_date``.
        goals (MutableSequence[google.ads.googleads.v21.common.types.MetricGoal]):
            The goals of this experiment.
        long_running_operation (str):
            Output only. The resource name of the
            long-running operation that can be used to poll
            for completion of experiment schedule or
            promote. The most recent long running operation
            is returned.

            This field is a member of `oneof`_ ``_long_running_operation``.
        promote_status (google.ads.googleads.v21.enums.types.AsyncActionStatusEnum.AsyncActionStatus):
            Output only. The status of the experiment
            promotion process.
        sync_enabled (bool):
            Immutable. Set to true if changes to base
            campaigns should be synced to the trial
            campaigns. Any changes made directly to trial
            campaigns will be preserved. This field can only
            be set when the experiment is being created.

            This field is a member of `oneof`_ ``_sync_enabled``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    experiment_id: int = proto.Field(
        proto.INT64,
        number=9,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=10,
    )
    description: str = proto.Field(
        proto.STRING,
        number=11,
    )
    suffix: str = proto.Field(
        proto.STRING,
        number=12,
    )
    type_: experiment_type.ExperimentTypeEnum.ExperimentType = proto.Field(
        proto.ENUM,
        number=13,
        enum=experiment_type.ExperimentTypeEnum.ExperimentType,
    )
    status: experiment_status.ExperimentStatusEnum.ExperimentStatus = (
        proto.Field(
            proto.ENUM,
            number=14,
            enum=experiment_status.ExperimentStatusEnum.ExperimentStatus,
        )
    )
    start_date: str = proto.Field(
        proto.STRING,
        number=15,
        optional=True,
    )
    end_date: str = proto.Field(
        proto.STRING,
        number=16,
        optional=True,
    )
    goals: MutableSequence[metric_goal.MetricGoal] = proto.RepeatedField(
        proto.MESSAGE,
        number=17,
        message=metric_goal.MetricGoal,
    )
    long_running_operation: str = proto.Field(
        proto.STRING,
        number=18,
        optional=True,
    )
    promote_status: (
        async_action_status.AsyncActionStatusEnum.AsyncActionStatus
    ) = proto.Field(
        proto.ENUM,
        number=19,
        enum=async_action_status.AsyncActionStatusEnum.AsyncActionStatus,
    )
    sync_enabled: bool = proto.Field(
        proto.BOOL,
        number=20,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "TrackingCodeTypeEnum",
    },
)


class TrackingCodeTypeEnum(proto.Message):
    r"""Container for enum describing the type of the generated tag
    snippets for tracking conversions.

    """

    class TrackingCodeType(proto.Enum):
        r"""The type of the generated tag snippets for tracking
        conversions.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            WEBPAGE (2):
                The snippet that is fired as a result of a
                website page loading.
            WEBPAGE_ONCLICK (3):
                The snippet contains a JavaScript function
                which fires the tag. This function is typically
                called from an onClick handler added to a link
                or button element on the page.
            CLICK_TO_CALL (4):
                For embedding on a mobile webpage. The
                snippet contains a JavaScript function which
                fires the tag.
            WEBSITE_CALL (5):
                The snippet that is used to replace the phone
                number on your website with a Google forwarding
                number for call tracking purposes.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        WEBPAGE = 2
        WEBPAGE_ONCLICK = 3
        CLICK_TO_CALL = 4
        WEBSITE_CALL = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "YoutubeVideoRegistrationErrorEnum",
    },
)


class YoutubeVideoRegistrationErrorEnum(proto.Message):
    r"""Container for enum describing YouTube video registration
    errors.

    """

    class YoutubeVideoRegistrationError(proto.Enum):
        r"""Enum describing YouTube video registration errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            VIDEO_NOT_FOUND (2):
                Video to be registered wasn't found.
            VIDEO_NOT_ACCESSIBLE (3):
                Video to be registered is not accessible (for
                example, private).
            VIDEO_NOT_ELIGIBLE (4):
                Video to be registered is not eligible (for
                example, mature content).
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        VIDEO_NOT_FOUND = 2
        VIDEO_NOT_ACCESSIBLE = 3
        VIDEO_NOT_ELIGIBLE = 4


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v19.enums.types import (
    listing_group_filter_custom_attribute_index,
)
from google.ads.googleads.v19.enums.types import (
    listing_group_filter_listing_source,
)
from google.ads.googleads.v19.enums.types import (
    listing_group_filter_product_category_level,
)
from google.ads.googleads.v19.enums.types import (
    listing_group_filter_product_channel,
)
from google.ads.googleads.v19.enums.types import (
    listing_group_filter_product_condition,
)
from google.ads.googleads.v19.enums.types import (
    listing_group_filter_product_type_level,
)
from google.ads.googleads.v19.enums.types import listing_group_filter_type_enum


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "AssetGroupListingGroupFilter",
        "ListingGroupFilterDimensionPath",
        "ListingGroupFilterDimension",
    },
)


class AssetGroupListingGroupFilter(proto.Message):
    r"""AssetGroupListingGroupFilter represents a listing group
    filter tree node in an asset group.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the asset group listing
            group filter. Asset group listing group filter resource name
            have the form:

            ``customers/{customer_id}/assetGroupListingGroupFilters/{asset_group_id}~{listing_group_filter_id}``
        asset_group (str):
            Immutable. The asset group which this asset
            group listing group filter is part of.
        id (int):
            Output only. The ID of the
            ListingGroupFilter.
        type_ (google.ads.googleads.v19.enums.types.ListingGroupFilterTypeEnum.ListingGroupFilterType):
            Immutable. Type of a listing group filter
            node.
        listing_source (google.ads.googleads.v19.enums.types.ListingGroupFilterListingSourceEnum.ListingGroupFilterListingSource):
            Immutable. The source of listings filtered by
            this listing group filter.
        case_value (google.ads.googleads.v19.resources.types.ListingGroupFilterDimension):
            Dimension value with which this listing group
            is refining its parent. Undefined for the root
            group.
        parent_listing_group_filter (str):
            Immutable. Resource name of the parent
            listing group subdivision. Null for the root
            listing group filter node.
        path (google.ads.googleads.v19.resources.types.ListingGroupFilterDimensionPath):
            Output only. The path of dimensions defining
            this listing group filter.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset_group: str = proto.Field(
        proto.STRING,
        number=2,
    )
    id: int = proto.Field(
        proto.INT64,
        number=3,
    )
    type_: (
        listing_group_filter_type_enum.ListingGroupFilterTypeEnum.ListingGroupFilterType
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=listing_group_filter_type_enum.ListingGroupFilterTypeEnum.ListingGroupFilterType,
    )
    listing_source: (
        listing_group_filter_listing_source.ListingGroupFilterListingSourceEnum.ListingGroupFilterListingSource
    ) = proto.Field(
        proto.ENUM,
        number=9,
        enum=listing_group_filter_listing_source.ListingGroupFilterListingSourceEnum.ListingGroupFilterListingSource,
    )
    case_value: "ListingGroupFilterDimension" = proto.Field(
        proto.MESSAGE,
        number=6,
        message="ListingGroupFilterDimension",
    )
    parent_listing_group_filter: str = proto.Field(
        proto.STRING,
        number=7,
    )
    path: "ListingGroupFilterDimensionPath" = proto.Field(
        proto.MESSAGE,
        number=8,
        message="ListingGroupFilterDimensionPath",
    )


class ListingGroupFilterDimensionPath(proto.Message):
    r"""The path defining of dimensions defining a listing group
    filter.

    Attributes:
        dimensions (MutableSequence[google.ads.googleads.v19.resources.types.ListingGroupFilterDimension]):
            Output only. The complete path of dimensions
            through the listing group filter hierarchy
            (excluding the root node) to this listing group
            filter.
    """

    dimensions: MutableSequence["ListingGroupFilterDimension"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="ListingGroupFilterDimension",
        )
    )


class ListingGroupFilterDimension(proto.Message):
    r"""Listing dimensions for the asset group listing group filter.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        product_category (google.ads.googleads.v19.resources.types.ListingGroupFilterDimension.ProductCategory):
            Category of a product offer.

            This field is a member of `oneof`_ ``dimension``.
        product_brand (google.ads.googleads.v19.resources.types.ListingGroupFilterDimension.ProductBrand):
            Brand of a product offer.

            This field is a member of `oneof`_ ``dimension``.
        product_channel (google.ads.googleads.v19.resources.types.ListingGroupFilterDimension.ProductChannel):
            Locality of a product offer.

            This field is a member of `oneof`_ ``dimension``.
        product_condition (google.ads.googleads.v19.resources.types.ListingGroupFilterDimension.ProductCondition):
            Condition of a product offer.

            This field is a member of `oneof`_ ``dimension``.
        product_custom_attribute (google.ads.googleads.v19.resources.types.ListingGroupFilterDimension.ProductCustomAttribute):
            Custom attribute of a product offer.

            This field is a member of `oneof`_ ``dimension``.
        product_item_id (google.ads.googleads.v19.resources.types.ListingGroupFilterDimension.ProductItemId):
            Item id of a product offer.

            This field is a member of `oneof`_ ``dimension``.
        product_type (google.ads.googleads.v19.resources.types.ListingGroupFilterDimension.ProductType):
            Type of a product offer.

            This field is a member of `oneof`_ ``dimension``.
        webpage (google.ads.googleads.v19.resources.types.ListingGroupFilterDimension.Webpage):
            Filters for URLs in a page feed and URLs from
            the advertiser web domain.

            This field is a member of `oneof`_ ``dimension``.
    """

    class ProductCategory(proto.Message):
        r"""One element of a category at a certain level. Top-level
        categories are at level 1, their children at level 2, and so on.
        We currently support up to 5 levels. The user must specify a
        dimension type that indicates the level of the category. All
        cases of the same subdivision must have the same dimension type
        (category level).


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            category_id (int):
                ID of the product category.

                This ID is equivalent to the google_product_category ID as
                described in this article:
                https://support.google.com/merchants/answer/6324436

                This field is a member of `oneof`_ ``_category_id``.
            level (google.ads.googleads.v19.enums.types.ListingGroupFilterProductCategoryLevelEnum.ListingGroupFilterProductCategoryLevel):
                Indicates the level of the category in the
                taxonomy.
        """

        category_id: int = proto.Field(
            proto.INT64,
            number=1,
            optional=True,
        )
        level: (
            listing_group_filter_product_category_level.ListingGroupFilterProductCategoryLevelEnum.ListingGroupFilterProductCategoryLevel
        ) = proto.Field(
            proto.ENUM,
            number=2,
            enum=listing_group_filter_product_category_level.ListingGroupFilterProductCategoryLevelEnum.ListingGroupFilterProductCategoryLevel,
        )

    class ProductBrand(proto.Message):
        r"""Brand of the product.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            value (str):
                String value of the product brand.

                This field is a member of `oneof`_ ``_value``.
        """

        value: str = proto.Field(
            proto.STRING,
            number=1,
            optional=True,
        )

    class ProductChannel(proto.Message):
        r"""Locality of a product offer.

        Attributes:
            channel (google.ads.googleads.v19.enums.types.ListingGroupFilterProductChannelEnum.ListingGroupFilterProductChannel):
                Value of the locality.
        """

        channel: (
            listing_group_filter_product_channel.ListingGroupFilterProductChannelEnum.ListingGroupFilterProductChannel
        ) = proto.Field(
            proto.ENUM,
            number=1,
            enum=listing_group_filter_product_channel.ListingGroupFilterProductChannelEnum.ListingGroupFilterProductChannel,
        )

    class ProductCondition(proto.Message):
        r"""Condition of a product offer.

        Attributes:
            condition (google.ads.googleads.v19.enums.types.ListingGroupFilterProductConditionEnum.ListingGroupFilterProductCondition):
                Value of the condition.
        """

        condition: (
            listing_group_filter_product_condition.ListingGroupFilterProductConditionEnum.ListingGroupFilterProductCondition
        ) = proto.Field(
            proto.ENUM,
            number=1,
            enum=listing_group_filter_product_condition.ListingGroupFilterProductConditionEnum.ListingGroupFilterProductCondition,
        )

    class ProductCustomAttribute(proto.Message):
        r"""Custom attribute of a product offer.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            value (str):
                String value of the product custom attribute.

                This field is a member of `oneof`_ ``_value``.
            index (google.ads.googleads.v19.enums.types.ListingGroupFilterCustomAttributeIndexEnum.ListingGroupFilterCustomAttributeIndex):
                Indicates the index of the custom attribute.
        """

        value: str = proto.Field(
            proto.STRING,
            number=1,
            optional=True,
        )
        index: (
            listing_group_filter_custom_attribute_index.ListingGroupFilterCustomAttributeIndexEnum.ListingGroupFilterCustomAttributeIndex
        ) = proto.Field(
            proto.ENUM,
            number=2,
            enum=listing_group_filter_custom_attribute_index.ListingGroupFilterCustomAttributeIndexEnum.ListingGroupFilterCustomAttributeIndex,
        )

    class ProductItemId(proto.Message):
        r"""Item id of a product offer.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            value (str):
                Value of the id.

                This field is a member of `oneof`_ ``_value``.
        """

        value: str = proto.Field(
            proto.STRING,
            number=1,
            optional=True,
        )

    class ProductType(proto.Message):
        r"""Type of a product offer.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            value (str):
                Value of the type.

                This field is a member of `oneof`_ ``_value``.
            level (google.ads.googleads.v19.enums.types.ListingGroupFilterProductTypeLevelEnum.ListingGroupFilterProductTypeLevel):
                Level of the type.
        """

        value: str = proto.Field(
            proto.STRING,
            number=1,
            optional=True,
        )
        level: (
            listing_group_filter_product_type_level.ListingGroupFilterProductTypeLevelEnum.ListingGroupFilterProductTypeLevel
        ) = proto.Field(
            proto.ENUM,
            number=2,
            enum=listing_group_filter_product_type_level.ListingGroupFilterProductTypeLevelEnum.ListingGroupFilterProductTypeLevel,
        )

    class Webpage(proto.Message):
        r"""Filters for URLs in a page feed and URLs from the advertiser
        web domain. Several root nodes with this dimension are allowed
        in an asset group and their conditions are considered in OR.

        Attributes:
            conditions (MutableSequence[google.ads.googleads.v19.resources.types.ListingGroupFilterDimension.WebpageCondition]):
                The webpage conditions are case sensitive and these are
                and-ed together when evaluated for filtering. All the
                conditions should be of same type. Example1: for URL1 =
                www.ads.google.com?ocid=1&euid=2 and URL2 =
                www.ads.google.com?ocid=1 and with "ocid" and "euid" as
                url_contains conditions, URL1 will be matched, but URL2 not.

                Example2 : If URL1 has Label1, Label2 and URL2 has Label2,
                Label3, then with Label1 and Label2 as custom_label
                conditions, URL1 will be matched but not URL2. With Label2
                as the only custom_label condition then both URL1 and URL2
                will be matched.
        """

        conditions: MutableSequence[
            "ListingGroupFilterDimension.WebpageCondition"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="ListingGroupFilterDimension.WebpageCondition",
        )

    class WebpageCondition(proto.Message):
        r"""Matching condition for URL filtering.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            custom_label (str):
                Filters the URLs in a page feed that have this custom label.
                A custom label can be added to a campaign by creating an
                AssetSet of type PAGE_FEED and linking it to the campaign
                using CampaignAssetSet.

                This field is a member of `oneof`_ ``condition``.
            url_contains (str):
                Filters the URLs in a page feed and the URLs
                from the advertiser web domain that contain this
                string.

                This field is a member of `oneof`_ ``condition``.
        """

        custom_label: str = proto.Field(
            proto.STRING,
            number=1,
            oneof="condition",
        )
        url_contains: str = proto.Field(
            proto.STRING,
            number=2,
            oneof="condition",
        )

    product_category: ProductCategory = proto.Field(
        proto.MESSAGE,
        number=10,
        oneof="dimension",
        message=ProductCategory,
    )
    product_brand: ProductBrand = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="dimension",
        message=ProductBrand,
    )
    product_channel: ProductChannel = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="dimension",
        message=ProductChannel,
    )
    product_condition: ProductCondition = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="dimension",
        message=ProductCondition,
    )
    product_custom_attribute: ProductCustomAttribute = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="dimension",
        message=ProductCustomAttribute,
    )
    product_item_id: ProductItemId = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="dimension",
        message=ProductItemId,
    )
    product_type: ProductType = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="dimension",
        message=ProductType,
    )
    webpage: Webpage = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="dimension",
        message=Webpage,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.common.types import criteria


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "CampaignBidModifier",
    },
)


class CampaignBidModifier(proto.Message):
    r"""Represents a bid-modifiable only criterion at the campaign
    level.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the campaign bid modifier.
            Campaign bid modifier resource names have the form:

            ``customers/{customer_id}/campaignBidModifiers/{campaign_id}~{criterion_id}``
        campaign (str):
            Output only. The campaign to which this
            criterion belongs.

            This field is a member of `oneof`_ ``_campaign``.
        criterion_id (int):
            Output only. The ID of the criterion to bid
            modify.
            This field is ignored for mutates.

            This field is a member of `oneof`_ ``_criterion_id``.
        bid_modifier (float):
            The modifier for the bid when the criterion
            matches.

            This field is a member of `oneof`_ ``_bid_modifier``.
        interaction_type (google.ads.googleads.v22.common.types.InteractionTypeInfo):
            Immutable. Criterion for interaction type.
            Only supported for search campaigns.

            This field is a member of `oneof`_ ``criterion``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=6,
        optional=True,
    )
    criterion_id: int = proto.Field(
        proto.INT64,
        number=7,
        optional=True,
    )
    bid_modifier: float = proto.Field(
        proto.DOUBLE,
        number=8,
        optional=True,
    )
    interaction_type: criteria.InteractionTypeInfo = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="criterion",
        message=criteria.InteractionTypeInfo,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.enums.types import month_of_year
from google.ads.googleads.v21.resources.types import invoice


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.services",
    marshal="google.ads.googleads.v21",
    manifest={
        "ListInvoicesRequest",
        "ListInvoicesResponse",
    },
)


class ListInvoicesRequest(proto.Message):
    r"""Request message for fetching the invoices of a given billing
    setup that were issued during a given month.

    Attributes:
        customer_id (str):
            Required. The ID of the customer to fetch
            invoices for.
        billing_setup (str):
            Required. The billing setup resource name of the requested
            invoices.

            ``customers/{customer_id}/billingSetups/{billing_setup_id}``
        issue_year (str):
            Required. The issue year to retrieve
            invoices, in yyyy format. Only invoices issued
            in 2019 or later can be retrieved.
        issue_month (google.ads.googleads.v21.enums.types.MonthOfYearEnum.MonthOfYear):
            Required. The issue month to retrieve
            invoices.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    billing_setup: str = proto.Field(
        proto.STRING,
        number=2,
    )
    issue_year: str = proto.Field(
        proto.STRING,
        number=3,
    )
    issue_month: month_of_year.MonthOfYearEnum.MonthOfYear = proto.Field(
        proto.ENUM,
        number=4,
        enum=month_of_year.MonthOfYearEnum.MonthOfYear,
    )


class ListInvoicesResponse(proto.Message):
    r"""Response message for
    [InvoiceService.ListInvoices][google.ads.googleads.v21.services.InvoiceService.ListInvoices].

    Attributes:
        invoices (MutableSequence[google.ads.googleads.v21.resources.types.Invoice]):
            The list of invoices that match the billing
            setup and time period.
    """

    invoices: MutableSequence[invoice.Invoice] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=invoice.Invoice,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v19.common.types import policy
from google.ads.googleads.v19.enums.types import ad_group_ad_primary_status
from google.ads.googleads.v19.enums.types import (
    ad_group_ad_primary_status_reason,
)
from google.ads.googleads.v19.enums.types import ad_group_ad_status
from google.ads.googleads.v19.enums.types import ad_strength as gage_ad_strength
from google.ads.googleads.v19.enums.types import (
    asset_automation_status as gage_asset_automation_status,
)
from google.ads.googleads.v19.enums.types import (
    asset_automation_type as gage_asset_automation_type,
)
from google.ads.googleads.v19.enums.types import policy_approval_status
from google.ads.googleads.v19.enums.types import policy_review_status
from google.ads.googleads.v19.resources.types import ad as gagr_ad


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "AdGroupAd",
        "AdGroupAdPolicySummary",
        "AdGroupAdAssetAutomationSetting",
    },
)


class AdGroupAd(proto.Message):
    r"""An ad group ad.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the ad. Ad group ad resource
            names have the form:

            ``customers/{customer_id}/adGroupAds/{ad_group_id}~{ad_id}``
        status (google.ads.googleads.v19.enums.types.AdGroupAdStatusEnum.AdGroupAdStatus):
            The status of the ad.
        ad_group (str):
            Immutable. The ad group to which the ad
            belongs.

            This field is a member of `oneof`_ ``_ad_group``.
        ad (google.ads.googleads.v19.resources.types.Ad):
            Immutable. The ad.
        policy_summary (google.ads.googleads.v19.resources.types.AdGroupAdPolicySummary):
            Output only. Policy information for the ad.
        ad_strength (google.ads.googleads.v19.enums.types.AdStrengthEnum.AdStrength):
            Output only. Overall ad strength for this ad
            group ad.
        action_items (MutableSequence[str]):
            Output only. A list of recommendations to
            improve the ad strength. For example, a
            recommendation could be "Try adding a few more
            unique headlines or unpinning some assets.".
        labels (MutableSequence[str]):
            Output only. The resource names of labels
            attached to this ad group ad.
        primary_status (google.ads.googleads.v19.enums.types.AdGroupAdPrimaryStatusEnum.AdGroupAdPrimaryStatus):
            Output only. Provides aggregated view into
            why an ad group ad is not serving or not serving
            optimally.
        primary_status_reasons (MutableSequence[google.ads.googleads.v19.enums.types.AdGroupAdPrimaryStatusReasonEnum.AdGroupAdPrimaryStatusReason]):
            Output only. Provides reasons for why an ad
            group ad is not serving or not serving
            optimally.
        ad_group_ad_asset_automation_settings (MutableSequence[google.ads.googleads.v19.resources.types.AdGroupAdAssetAutomationSetting]):
            Settings that control the types of asset
            automation. See the AssetAutomationTypeEnum
            documentation for the default opt in/out
            behavior of each type.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    status: ad_group_ad_status.AdGroupAdStatusEnum.AdGroupAdStatus = (
        proto.Field(
            proto.ENUM,
            number=3,
            enum=ad_group_ad_status.AdGroupAdStatusEnum.AdGroupAdStatus,
        )
    )
    ad_group: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    ad: gagr_ad.Ad = proto.Field(
        proto.MESSAGE,
        number=5,
        message=gagr_ad.Ad,
    )
    policy_summary: "AdGroupAdPolicySummary" = proto.Field(
        proto.MESSAGE,
        number=6,
        message="AdGroupAdPolicySummary",
    )
    ad_strength: gage_ad_strength.AdStrengthEnum.AdStrength = proto.Field(
        proto.ENUM,
        number=7,
        enum=gage_ad_strength.AdStrengthEnum.AdStrength,
    )
    action_items: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=13,
    )
    labels: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=10,
    )
    primary_status: (
        ad_group_ad_primary_status.AdGroupAdPrimaryStatusEnum.AdGroupAdPrimaryStatus
    ) = proto.Field(
        proto.ENUM,
        number=16,
        enum=ad_group_ad_primary_status.AdGroupAdPrimaryStatusEnum.AdGroupAdPrimaryStatus,
    )
    primary_status_reasons: MutableSequence[
        ad_group_ad_primary_status_reason.AdGroupAdPrimaryStatusReasonEnum.AdGroupAdPrimaryStatusReason
    ] = proto.RepeatedField(
        proto.ENUM,
        number=17,
        enum=ad_group_ad_primary_status_reason.AdGroupAdPrimaryStatusReasonEnum.AdGroupAdPrimaryStatusReason,
    )
    ad_group_ad_asset_automation_settings: MutableSequence[
        "AdGroupAdAssetAutomationSetting"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=18,
        message="AdGroupAdAssetAutomationSetting",
    )


class AdGroupAdPolicySummary(proto.Message):
    r"""Contains policy information for an ad.

    Attributes:
        policy_topic_entries (MutableSequence[google.ads.googleads.v19.common.types.PolicyTopicEntry]):
            Output only. The list of policy findings for
            this ad.
        review_status (google.ads.googleads.v19.enums.types.PolicyReviewStatusEnum.PolicyReviewStatus):
            Output only. Where in the review process this
            ad is.
        approval_status (google.ads.googleads.v19.enums.types.PolicyApprovalStatusEnum.PolicyApprovalStatus):
            Output only. The overall approval status of
            this ad, calculated based on the status of its
            individual policy topic entries.
    """

    policy_topic_entries: MutableSequence[policy.PolicyTopicEntry] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=policy.PolicyTopicEntry,
        )
    )
    review_status: (
        policy_review_status.PolicyReviewStatusEnum.PolicyReviewStatus
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=policy_review_status.PolicyReviewStatusEnum.PolicyReviewStatus,
    )
    approval_status: (
        policy_approval_status.PolicyApprovalStatusEnum.PolicyApprovalStatus
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=policy_approval_status.PolicyApprovalStatusEnum.PolicyApprovalStatus,
    )


class AdGroupAdAssetAutomationSetting(proto.Message):
    r"""Asset automation setting for an AdGroupAd.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        asset_automation_type (google.ads.googleads.v19.enums.types.AssetAutomationTypeEnum.AssetAutomationType):
            The asset automation type that this setting
            configures.

            This field is a member of `oneof`_ ``_asset_automation_type``.
        asset_automation_status (google.ads.googleads.v19.enums.types.AssetAutomationStatusEnum.AssetAutomationStatus):
            The opt-in/out status for the specified asset
            automation type.

            This field is a member of `oneof`_ ``_asset_automation_status``.
    """

    asset_automation_type: (
        gage_asset_automation_type.AssetAutomationTypeEnum.AssetAutomationType
    ) = proto.Field(
        proto.ENUM,
        number=1,
        optional=True,
        enum=gage_asset_automation_type.AssetAutomationTypeEnum.AssetAutomationType,
    )
    asset_automation_status: (
        gage_asset_automation_status.AssetAutomationStatusEnum.AssetAutomationStatus
    ) = proto.Field(
        proto.ENUM,
        number=2,
        optional=True,
        enum=gage_asset_automation_status.AssetAutomationStatusEnum.AssetAutomationStatus,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "LocalServicesLeadConversationTypeEnum",
    },
)


class LocalServicesLeadConversationTypeEnum(proto.Message):
    r"""Container for enum describing possible types of lead
    conversation.

    """

    class ConversationType(proto.Enum):
        r"""Possible types of lead conversation.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            EMAIL (2):
                Email lead conversation.
            MESSAGE (3):
                Message lead conversation.
            PHONE_CALL (4):
                Phone call lead conversation.
            SMS (5):
                SMS lead conversation.
            BOOKING (6):
                Booking lead conversation.
            WHATSAPP (7):
                WhatsApp lead conversation.
            ADS_API (8):
                Lead conversation created through Google Ads
                API.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        EMAIL = 2
        MESSAGE = 3
        PHONE_CALL = 4
        SMS = 5
        BOOKING = 6
        WHATSAPP = 7
        ADS_API = 8


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.common.types import criteria
from google.ads.googleads.v22.enums.types import criterion_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "SharedCriterion",
    },
)


class SharedCriterion(proto.Message):
    r"""A criterion belonging to a shared set.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the shared criterion. Shared
            set resource names have the form:

            ``customers/{customer_id}/sharedCriteria/{shared_set_id}~{criterion_id}``
        shared_set (str):
            Immutable. The shared set to which the shared
            criterion belongs.

            This field is a member of `oneof`_ ``_shared_set``.
        criterion_id (int):
            Output only. The ID of the criterion.

            This field is ignored for mutates.

            This field is a member of `oneof`_ ``_criterion_id``.
        type_ (google.ads.googleads.v22.enums.types.CriterionTypeEnum.CriterionType):
            Output only. The type of the criterion.
        keyword (google.ads.googleads.v22.common.types.KeywordInfo):
            Immutable. Keyword.

            This field is a member of `oneof`_ ``criterion``.
        youtube_video (google.ads.googleads.v22.common.types.YouTubeVideoInfo):
            Immutable. YouTube Video.

            This field is a member of `oneof`_ ``criterion``.
        youtube_channel (google.ads.googleads.v22.common.types.YouTubeChannelInfo):
            Immutable. YouTube Channel.

            This field is a member of `oneof`_ ``criterion``.
        placement (google.ads.googleads.v22.common.types.PlacementInfo):
            Immutable. Placement.

            This field is a member of `oneof`_ ``criterion``.
        mobile_app_category (google.ads.googleads.v22.common.types.MobileAppCategoryInfo):
            Immutable. Mobile App Category.

            This field is a member of `oneof`_ ``criterion``.
        mobile_application (google.ads.googleads.v22.common.types.MobileApplicationInfo):
            Immutable. Mobile application.

            This field is a member of `oneof`_ ``criterion``.
        brand (google.ads.googleads.v22.common.types.BrandInfo):
            Immutable. Brand.

            This field is a member of `oneof`_ ``criterion``.
        webpage (google.ads.googleads.v22.common.types.WebpageInfo):
            Immutable. Webpage.

            This field is a member of `oneof`_ ``criterion``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    shared_set: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )
    criterion_id: int = proto.Field(
        proto.INT64,
        number=11,
        optional=True,
    )
    type_: criterion_type.CriterionTypeEnum.CriterionType = proto.Field(
        proto.ENUM,
        number=4,
        enum=criterion_type.CriterionTypeEnum.CriterionType,
    )
    keyword: criteria.KeywordInfo = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="criterion",
        message=criteria.KeywordInfo,
    )
    youtube_video: criteria.YouTubeVideoInfo = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="criterion",
        message=criteria.YouTubeVideoInfo,
    )
    youtube_channel: criteria.YouTubeChannelInfo = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="criterion",
        message=criteria.YouTubeChannelInfo,
    )
    placement: criteria.PlacementInfo = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="criterion",
        message=criteria.PlacementInfo,
    )
    mobile_app_category: criteria.MobileAppCategoryInfo = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="criterion",
        message=criteria.MobileAppCategoryInfo,
    )
    mobile_application: criteria.MobileApplicationInfo = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="criterion",
        message=criteria.MobileApplicationInfo,
    )
    brand: criteria.BrandInfo = proto.Field(
        proto.MESSAGE,
        number=12,
        oneof="criterion",
        message=criteria.BrandInfo,
    )
    webpage: criteria.WebpageInfo = proto.Field(
        proto.MESSAGE,
        number=13,
        oneof="criterion",
        message=criteria.WebpageInfo,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v20.resources.types import google_ads_field


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.services",
    marshal="google.ads.googleads.v20",
    manifest={
        "GetGoogleAdsFieldRequest",
        "SearchGoogleAdsFieldsRequest",
        "SearchGoogleAdsFieldsResponse",
    },
)


class GetGoogleAdsFieldRequest(proto.Message):
    r"""Request message for
    [GoogleAdsFieldService.GetGoogleAdsField][google.ads.googleads.v20.services.GoogleAdsFieldService.GetGoogleAdsField].

    Attributes:
        resource_name (str):
            Required. The resource name of the field to
            get.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class SearchGoogleAdsFieldsRequest(proto.Message):
    r"""Request message for
    [GoogleAdsFieldService.SearchGoogleAdsFields][google.ads.googleads.v20.services.GoogleAdsFieldService.SearchGoogleAdsFields].

    Attributes:
        query (str):
            Required. The query string.
        page_token (str):
            Token of the page to retrieve. If not specified, the first
            page of results will be returned. Use the value obtained
            from ``next_page_token`` in the previous response in order
            to request the next page of results.
        page_size (int):
            Number of elements to retrieve in a single
            page. When too large a page is requested, the
            server may decide to further limit the number of
            returned resources.
    """

    query: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=3,
    )


class SearchGoogleAdsFieldsResponse(proto.Message):
    r"""Response message for
    [GoogleAdsFieldService.SearchGoogleAdsFields][google.ads.googleads.v20.services.GoogleAdsFieldService.SearchGoogleAdsFields].

    Attributes:
        results (MutableSequence[google.ads.googleads.v20.resources.types.GoogleAdsField]):
            The list of fields that matched the query.
        next_page_token (str):
            Pagination token used to retrieve the next page of results.
            Pass the content of this string as the ``page_token``
            attribute of the next request. ``next_page_token`` is not
            returned for the last page.
        total_results_count (int):
            Total number of results that match the query
            ignoring the LIMIT clause.
    """

    @property
    def raw_page(self):
        return self

    results: MutableSequence[google_ads_field.GoogleAdsField] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=google_ads_field.GoogleAdsField,
        )
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    total_results_count: int = proto.Field(
        proto.INT64,
        number=3,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "LocalServicesInsuranceRejectionReasonEnum",
    },
)


class LocalServicesInsuranceRejectionReasonEnum(proto.Message):
    r"""Container for enum describing the rejection reason of a local
    services insurance verification artifact.

    """

    class LocalServicesInsuranceRejectionReason(proto.Enum):
        r"""Enums describing possible rejection reasons of a local
        services insurance verification artifact.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            BUSINESS_NAME_MISMATCH (2):
                Business name doesn't match business name for
                the Local Services Ad.
            INSURANCE_AMOUNT_INSUFFICIENT (3):
                Insurance amount doesn't meet requirement
                listed in the legal summaries documentation for
                that geographic + category ID combination.
            EXPIRED (4):
                Insurance document is expired.
            NO_SIGNATURE (5):
                Insurance document is missing a signature.
            NO_POLICY_NUMBER (6):
                Insurance document is missing a policy
                number.
            NO_COMMERCIAL_GENERAL_LIABILITY (7):
                Commercial General Liability(CGL) box is not
                marked in the insurance document.
            EDITABLE_FORMAT (8):
                Insurance document is in an editable format.
            CATEGORY_MISMATCH (9):
                Insurance document does not cover insurance
                for a particular category.
            MISSING_EXPIRATION_DATE (10):
                Insurance document is missing an expiration
                date.
            POOR_QUALITY (11):
                Insurance document is poor quality - blurry
                images, illegible, etc...
            POTENTIALLY_EDITED (12):
                Insurance document is suspected of being
                edited.
            WRONG_DOCUMENT_TYPE (13):
                Insurance document not accepted. For example,
                documents of insurance proposals, but missing
                coverages are not accepted.
            NON_FINAL (14):
                Insurance document is not final.
            OTHER (15):
                Insurance document has another flaw not
                listed explicitly.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        BUSINESS_NAME_MISMATCH = 2
        INSURANCE_AMOUNT_INSUFFICIENT = 3
        EXPIRED = 4
        NO_SIGNATURE = 5
        NO_POLICY_NUMBER = 6
        NO_COMMERCIAL_GENERAL_LIABILITY = 7
        EDITABLE_FORMAT = 8
        CATEGORY_MISMATCH = 9
        MISSING_EXPIRATION_DATE = 10
        POOR_QUALITY = 11
        POTENTIALLY_EDITED = 12
        WRONG_DOCUMENT_TYPE = 13
        NON_FINAL = 14
        OTHER = 15


__all__ = tuple(sorted(__protobuf__.manifest))
